/*
 * (c) Copyright Adrian Pop, adrpo@ida.liu.se, 2006-02-21
 * All Rights Reserved.
 */
package org.pelab.rml.editor;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.MarkerUtilities;
import org.eclipse.ui.texteditor.TextEditorAction;

import org.pelab.rml.RMLPlugin;
import org.pelab.rml.editor.parser.CompilerMessageParser;
import org.pelab.rml.editor.parser.RMLCompilerMessageParser;
import org.pelab.rml.preferences.RMLPreferencePage;
import org.pelab.rml.views.RMLConsoleView;


/**
 * Class that defines the action of compiling the current C Sharp file
 */

public class RMLCompilerAction extends TextEditorAction {

	public static final String EXE = "exe"; 
	public static final String WINEXE = "winexe"; 
	public static final String LIBRARY = "library";
	public static final String MODULE = "module"; 
	
	public String compilerOutput = "";
	

	private static RMLCompilerAction instance = new RMLCompilerAction();

	/**
	 * Constructs and updates the action.
	 */
	private RMLCompilerAction() {
		super(RMLPlugin.getDefault().getResourceBundle(), "CompilerAction.", null);
		update();
	}

	public static RMLCompilerAction getInstance() {
		return instance;
	}
	
	class StreamGobbler extends Thread
	{
	    InputStream is;
	    String type;
	    
	    StreamGobbler(InputStream is, String type)
	    {
	        this.is = is;
	        this.type = type;
	    }
	    
	    public void run()
	    {
	        try
	        {
	            InputStreamReader isr = new InputStreamReader(is);
	            BufferedReader br = new BufferedReader(isr);
	            String line=null;	            
	            while ( (line = br.readLine()) != null)
	            {
	            	compilerOutput += line + "\n";
	            }
	            } catch (IOException ioe)
	              {
	                ioe.printStackTrace();  
	              }
	    }
	}
		
	/**
	 * Code called when the action is fired.
	 */
	public void run() 
	{
		RMLConsoleView console = RMLPlugin.getRMLConsole();

		try
		{
			console.showBusy(true);
			if (getRMLCompilerLocation() == null ) {
				MessageBox infoBox = new MessageBox(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), SWT.ICON_WARNING | SWT.OK);
				infoBox.setMessage(RMLPlugin.getResourceString("CompilerAction.messageBoxContent"));
				infoBox.setText(RMLPlugin.getResourceString("CompilerAction.messageBoxTitle"));
				infoBox.open();
				return;
			}
			IFile fileToCompile = findCurrentRMLFile();
			if (fileToCompile == null) 
			{
				// should never happen
				System.err.println("Error : no file in the editor");
				// should throw an exception
				return;
			}

			String command = buildCommand(fileToCompile);
			String filePath = fileToCompile.getLocation().toOSString();
			String fileFolderPath =	filePath.substring(0, filePath.length() - fileToCompile.getName().length());			
			
			Runtime r = Runtime.getRuntime();

			Process p = r.exec(command.toString(), null, new File(fileFolderPath));
			
			compilerOutput = "";

            // any output?
			StreamGobbler errorGobbler = new StreamGobbler(p.getErrorStream(), "ERROR");            
            StreamGobbler outputGobbler = new StreamGobbler(p.getInputStream(), "OUTPUT");
            
            // kick them off
            errorGobbler.start();
            outputGobbler.start();
            
            // any error???
            int exitVal = p.waitFor();
            if (exitVal != 0) compilerOutput = "Running the compiler:" + command.toString() + " failed!\n";
			// parse the buffer to find the errors and create markers
            console.getConsole().clearConsole();
			console.getConsole().getDocument().set(compilerOutput);
			createMarkers(compilerOutput, fileToCompile);	            	
			// And refresh the compilation unit folder
			fileToCompile.getParent().refreshLocal(IResource.DEPTH_ONE, null);
			console.showBusy(false);
		}
		catch(InterruptedException e)
		{
			console.showBusy(false);
			e.printStackTrace();
		}
		catch(IOException e)
		{
			console.showBusy(false);
			e.printStackTrace();
		}			
		catch(CoreException e)
		{
			console.showBusy(false);
			e.printStackTrace();
		}			
		catch (Exception e) 
		{
			console.showBusy(false);
			e.printStackTrace();
			// $$$ do something here !			
		}
	}


	/**
	 * Finds the file that's currently opened in the RML Text Editor
	 */
	protected IFile findCurrentRMLFile() 
	{
		ITextEditor editor = getTextEditor();

		IEditorInput editorInput = null;
		if (editor != null) 
		{
			editorInput = editor.getEditorInput();
		}

		if (editorInput instanceof IFileEditorInput)
			return ((IFileEditorInput) editorInput).getFile();
		
		return null;
	}

	/**
	 * Builds the command for compiling the current file. <br/>
	 * It uses the persistent properties of the file to know the
	 * compile-time arguments.
	 */
	protected String buildCommand(IFile fileToCompile) {
		StringBuffer command = new StringBuffer(getRMLCompilerLocation());
		command.append(" ");
		command.append(fileToCompile.getName());
		return command.toString();
	}	


	/**
	 * Creates a string buffer from the given input stream
	 */
	protected String getStringFromStream(BufferedInputStream stream) throws IOException {
		StringBuffer buffer = new StringBuffer();
		byte[] b = new byte[1];
		int finished = 1;
		while (finished > 0 && stream.available() > 0) 
		{
			finished = stream.read(b);
			if (finished > 0) {
				String current = new String(b, 0, finished);
				buffer.append(current);
			}
		}
		stream.close();
		return buffer.toString();
	}
	
	
	/**
	 * Create markers according to the compiler output
	 */
	protected void createMarkers(String output, IFile file) throws CoreException {
		// first delete all the previous markers
		file.deleteMarkers(IMarker.PROBLEM, false, 0);
		int depth = IResource.DEPTH_INFINITE;
		try 
		{
			file.getProject().deleteMarkers(IMarker.PROBLEM, true, depth);
		} 
		catch (CoreException e) 
		{
		      // something went wrong
		}
		CompilerMessageParser lc_compilerMessageParser = getCompilerMessageParser();
		lc_compilerMessageParser.parseCompilerMessage(file, output);
	}


	/**
     * @return
     */
    private CompilerMessageParser getCompilerMessageParser() {
    	String lc_compilerType = RMLPlugin.getDefault().getPreferenceStore().getString(RMLPreferencePage.RML_COMPILER_TYPE);
    	if(RMLPreferencePage.RML.equals(lc_compilerType))
    	    return new RMLCompilerMessageParser();
    	return new RMLCompilerMessageParser();
    }

    /**
	 * @return Returns the RMLCompilerLocation.
	 */
	public static String getRMLCompilerLocation() {
		String lc_compilerLocation = RMLPlugin.getDefault().getPreferenceStore().getString(RMLPreferencePage.RML_COMPILER_PATH);
		lc_compilerLocation = "".equals(lc_compilerLocation) ? null : lc_compilerLocation;
		return lc_compilerLocation;
		
	}
}