package org.pelab.rml.editor;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextDoubleClickStrategy;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.swt.graphics.RGB;

import org.pelab.rml.editor.rml.RMLCompletionProcessor;
import org.pelab.rml.editor.rml.RMLDoubleClickStrategy;
import org.pelab.rml.editor.rml.RMLPartitionScanner;
import org.pelab.rml.editor.doc.RMLCodeScanner;
import org.pelab.rml.editor.doc.RMLDocCompletionProcessor;
import org.pelab.rml.editor.doc.RMLDocScanner;
import org.pelab.rml.editor.utils.RMLColorManager;

public class RMLConfiguration extends SourceViewerConfiguration {
    private RMLDoubleClickStrategy doubleClickStrategy;
    private RMLDocScanner tagScanner;
    private RMLCodeScanner scanner;
    private RMLColorManager colorManager;

    public RMLConfiguration(RMLColorManager colorManager) {
        this.colorManager = colorManager;
    }

    public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
        return new String[] { IDocument.DEFAULT_CONTENT_TYPE,
                RMLPartitionScanner.RML_DOC,
                RMLPartitionScanner.RML_MULTILINE_COMMENT };
    }

    public ITextDoubleClickStrategy getDoubleClickStrategy(
            ISourceViewer sourceViewer, String contentType) {
        if (doubleClickStrategy == null)
            doubleClickStrategy = new RMLDoubleClickStrategy();
        return doubleClickStrategy;
    }

    protected RMLCodeScanner getRMLScanner() {
        if (scanner == null) {
            scanner = new RMLCodeScanner(colorManager);
            //scanner.setDefaultReturnToken(new Token(new TextAttribute(
            //        colorManager.getColor(IXMLColorConstants.DEFAULT))));
        }
        return scanner;
    }

    protected RMLDocScanner getRMLDocScanner() {
        if (tagScanner == null) {
            tagScanner = new RMLDocScanner(colorManager);
            //tagScanner.setDefaultReturnToken(new Token(new TextAttribute(
            //        colorManager.getColor(IXMLColorConstants.TAG))));
        }
        return tagScanner;
    }

    
    
    

    public IPresentationReconciler getPresentationReconciler(
            ISourceViewer sourceViewer) {
	PresentationReconciler reconciler= new PresentationReconciler();

	// rule for default text
	DefaultDamagerRepairer dr= new DefaultDamagerRepairer(getRMLScanner());
	reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
	reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

	// rule for multiline comments
	// We jsut need a scanner that does nothing but returns a token with the corrresponding text attributes
	RuleBasedScanner multiLineScanner = new RuleBasedScanner();
	multiLineScanner.setDefaultReturnToken(new Token(new TextAttribute(colorManager.getColor(RMLColorManager.MULTI_LINE_COMMENT))));
	dr= new DefaultDamagerRepairer(multiLineScanner);
	reconciler.setDamager(dr, RMLPartitionScanner.RML_MULTILINE_COMMENT);
	reconciler.setRepairer(dr, RMLPartitionScanner.RML_MULTILINE_COMMENT);

	// rule for c sharp comments for documentation
	dr= new DefaultDamagerRepairer(getRMLDocScanner());
	reconciler.setDamager(dr, RMLPartitionScanner.RML_DOC);
	reconciler.setRepairer(dr, RMLPartitionScanner.RML_DOC);
	
	return reconciler;
}

    /* (non-Javadoc)
     * Method declared on SourceViewerConfiguration
     */
    public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {
    
    	ContentAssistant assistant= new ContentAssistant();
    	assistant.setContentAssistProcessor(new RMLCompletionProcessor(), IDocument.DEFAULT_CONTENT_TYPE);
    	assistant.setContentAssistProcessor(new RMLDocCompletionProcessor(), RMLPartitionScanner.RML_DOC);
    
    	assistant.enableAutoActivation(true);
    	assistant.setAutoActivationDelay(500);
    	assistant.setProposalPopupOrientation(IContentAssistant.PROPOSAL_OVERLAY);
    	assistant.setContextInformationPopupOrientation(IContentAssistant.CONTEXT_INFO_ABOVE);
    	assistant.setContextInformationPopupBackground(colorManager.getColor(new RGB(150, 150, 0)));
    
    	return assistant;
    }

    /* (non-Javadoc)
     * Method declared on SourceViewerConfiguration
     */
    public String getDefaultPrefix(ISourceViewer sourceViewer, String contentType) {
    	return (IDocument.DEFAULT_CONTENT_TYPE.equals(contentType) ? "//" : null); 
    }

    /* (non-Javadoc)
     * Method declared on SourceViewerConfiguration
     */
    public String[] getIndentPrefixes(ISourceViewer sourceViewer, String contentType) {
    	return new String[] { "\t", "    " };
    }

    /* (non-Javadoc)
     * Method declared on SourceViewerConfiguration
     */
    public int getTabWidth(ISourceViewer sourceViewer) {
    	return 4;
    }
    
    


}