/*
 * This file is part of RML Development Tooling.
 *
 * Copyright (c) 2006, Adrian Pop, adrpo@ida.liu.se
 * All Rights Reserved
 */

package org.pelab.rml.editor;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import java.util.Comparator;
import java.util.Collections;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * @author Adrian Pop
  */
public class RMLRuleScanner extends RuleBasedScanner 
{

	private static Color CODE_COLOR
		= new Color(Display.getCurrent(), new RGB(0, 0, 0));
	private static Color COMMENT_COLOR 
		= new Color(Display.getCurrent(), new RGB(100, 100, 100));
	private static Color STRING_COLOR 
		= new Color(Display.getCurrent(), new RGB(139, 0, 139));
	private static Color CHAR_COLOR 
		= new Color(Display.getCurrent(), new RGB(139, 10, 139));
	private static Color KEYWORD_COLOR 
		= new Color(Display.getCurrent(), new RGB(139, 0, 0));
	private static Color TYPE_COLOR 
		= new Color(Display.getCurrent(), new RGB(0, 0, 139));
	private static Color TYPEVAR_COLOR 
		= new Color(Display.getCurrent(), new RGB(10, 10, 139));
	private static Color DATATYPECONSTRUCTOR_COLOR 
		= new Color(Display.getCurrent(), new RGB(0, 0, 255));
	private static Color NUMBER_COLOR 
		= new Color(Display.getCurrent(), new RGB(255, 100, 139));
	private static Color BUILTIN_COLOR 
		= new Color(Display.getCurrent(), new RGB(0, 139, 139));
	private static Color DASHES_COLOR 
		= new Color(Display.getCurrent(), new RGB(255, 0, 139));
	private static Color AMPERSANT_COLOR 
		= new Color(Display.getCurrent(), new RGB(255, 0, 100));
	private static Color ERROR_COLOR 
		= new Color(Display.getCurrent(), new RGB(255, 0, 0));

	/**
	 * Whitespace detector
	 */
	static private class WhiteSpace implements IWhitespaceDetector
	{
		public boolean isWhitespace(char c)
		{
			return Character.isWhitespace(c);
		}
	}
	
	/**
	 * RML keyword detector
	 */
	static private class RMLKeywordDetector implements IWordDetector
	{
		/* keywords list has to be alphabetized for this to work properly 
		 * copied from parser.y
		 *  ("and",	Tokens.AND), ("as",	Tokens.AS), ("axiom",	Tokens.AXIOM),
		 *  ("datatype",Tokens.DATATYPE), ("default",	Tokens.DEFAULT),
		 *  ("end",	Tokens.END), ("eqtype",	Tokens.EQTYPE),
		 *  ("fail",	Tokens.FAIL),
		 *  ("let",	Tokens.LET),
		 *  ("module",	Tokens.MODULE),
		 *  ("not",	Tokens.NOT),
		 *  ("of",	Tokens.OF),
		 *  ("relation",Tokens.RELATION), ("rule",	Tokens.RULE),
		 *  ("type",	Tokens.TYPE),
		 *  ("val",	Tokens.VAL),
		 *  ("with",	Tokens.WITH), ("withtype",Tokens.WITHTYPE)
		 */
		static public String[] keywords = {
			"and","as","axiom",
			"datatype","default",
			"end","eqtype","fail","false",
			"let","module",
			"nil","not","of","relation","rule",
			"true","type",
			"val","with",
			"withtype"
		};
		
		public RMLKeywordDetector() 
		{
		}

		public boolean isWordStart(char c) 
		{
			if (c == '_') { return false; }
			return Character.isJavaIdentifierStart(c);
		}

		public boolean isWordPart(char c) 
		{
			if (c == '\'') { return true; }
			return Character.isJavaIdentifierPart(c);
		}
	}

	/**
	 * RML type variable detector
	 */
	static private class RMLTypeVariableDetector extends RMLKeywordDetector implements IWordDetector 
	{

		public RMLTypeVariableDetector() 
		{
		}

		public boolean isWordStart(char c) 
		{
			if (c == '\'') { return true; }
			return false;
		}

		public boolean isWordPart(char c) 
		{
			if (c == '\'') { return true; }
			return Character.isJavaIdentifierPart(c);
		}
	}
	
	/**
	 * RML type detector
	 */
	static private class RMLTypeDetector extends RMLTypeVariableDetector implements IWordDetector 
	{
		/* keywords list has to be alphabetized for this to work properly */
		static public String[] keywords = 
		{
			"array","bool","char","int","list","option","real","string","vector"
		};

		public RMLTypeDetector() 
		{
		}

		public boolean isWordStart(char c) 
		{
			if (c == '_') { return false; }
			if (c == '\'') { return true; }
			return Character.isJavaIdentifierStart(c);
		}

		public boolean isWordPart(char c) 
		{
			if (c == '\'') { return true; }
			return Character.isJavaIdentifierPart(c);
		}
	}

	/**
	 * RML builtin relation detector
	 */
	static private class RMLBuiltinRelationsDetector extends RMLTypeDetector implements IWordDetector 
	{
		/* keywords list has to be alphabetized for this to work properly */
		static public String[] keywords = 
		{
		      /* booleans */
			  "bool_and","bool_not","bool_or",
			  "boolAnd","boolNot","boolOr",
			  /* characters */
			  "charInt","stringCharInt",
			  "char_int","string_char_int",
			  /* integers */
			  "int_abs","int_add","int_char","int_string_char","int_div","int_eq","int_ge","int_gt",
			  "int_le","int_lt","int_max","int_min","int_mod","int_mul","int_ne","int_neg","int_real",
			  "int_string","int_sub",
			  "intAbs","intAdd","intChar","intStringChar","intDiv","intEq","intGe","intGt","intLe","intLt",
			  "intMax","intMin","intMod","intMul","intNe","intNeg","intReal","intString","intSub",
			  /* lists */
			  "list_append","list_delete","list_length","list_member","list_nth","list_reverse",
			  "list_string","string_char_list_string","list_vector","list_array",	  
			  "listAppend","listDelete","listLength","listMember","listNth","listReverse",
			  "listString","stringCharListString","listVector","listArray",
			  /* logical variables */
			  "lvarGet","lvarNew","lvarSet",
			  "lvar_get","lvar_new","lvar_set",
			  /* reals */
			  "real_abs","real_add","real_atan","real_cos","real_div","real_eq","real_exp","real_floor",
			  "real_ge","real_gt","real_int","real_le","real_ln","real_lt","real_max","real_min",
			  "real_mod","real_mul","real_ne","real_neg","real_pow","real_sin","real_sqrt",
			  "real_string","real_sub",
			  "realAbs","realAdd","realAtan","realCos","realDiv","realEq","realExp","realFloor","realGe",
			  "realGt","realInt","realLe","realLn","realLt","realMax","realMin","realMod","realMul",
			  "realNe","realNeg","realPow","realSin","realSqrt","realString","realSub",
			  /* strings */
			  "string_append","string_int","string_length","string_list","string_list_string_char",
			  "string_nth","string_nth_string_char","string_get","string_get_string_char",
			  "string_setnth","string_setnth_string_char","string_update","string_update_string_char",
			  "stringAppend","stringInt","stringLength","stringList","stringListStringChar","stringNth",
			  "stringNthStringChar","stringGet","stringGetStringChar","stringSetNth","stringSetNthStringChar",
			  "stringUpdate","stringUpdateStringChar",
			  /* immutable vectors */
			  "vector_length","vector_list","vector_nth","vector_get","vector_setnth",
			  "vector_update","vector_create","vector_add","vector_array","vector_copy",
			  "vectorLength","vectorList","vectorNth","vectorGet","vectorSetNth",
			  "vectorUpdate","vectorCreate","vectorAdd","vectorArray","vectorCopy",      
			  "arrayLength","arrayList","arrayNth","arrayGet","arraySetNth","arrayUpdate",	  
			  "arrayCreate","arrayAdd","arrayVector","arrayCopy",
			  /* mutable arrays */      
			  "array_length","array_list","array_nth","array_get","array_setnth",
			  "array_update","array_create","array_add","array_vector","array_copy",      
			  /* misc */
			  "clock","print","tick",
			  /* debug */
			  "debug","debug_push_vars","debug_print", "debug_push_in01", "debug_push_in02", "debug_push_in03", "debug_push_in04", 
			  "debug_push_in05", "debug_push_in06", "debug_push_in07", "debug_push_in08", "debug_push_in09", "debug_push_in10", 
			  "debug_push_in11", "debug_push_in12", "debug_push_in13", "debug_push_in14", "debug_push_in15", "debug_push_in16", 
			  "debug_push_out01", "debug_push_out02", "debug_push_out03", "debug_push_out04", 
			  "debug_push_out05", "debug_push_out06", "debug_push_out07", "debug_push_out08", "debug_push_out09", "debug_push_out10", 
			  "debug_push_out11", "debug_push_out12", "debug_push_out13", "debug_push_out14", "debug_push_out15", "debug_push_out16",
			  "cons"
		};

		public RMLBuiltinRelationsDetector() 
		{
		}
				
		static public Vector<String> getSortedKeywordsList()
		{
			Vector<String> keys = new Vector<String>(keywords.length);
			for (int i=0; i< keywords.length;i++) 
			{
				keys.add(keywords[i]);
				keys.add("RML."+keywords[i]);
			}
			keys.add("RML.list");
			keys.add("RML.nil");			
			Collections.sort(keys, new Comparator<String>() {
				   public int compare(String o1, String o2) {
					      return o1.compareTo(o2);
					   }
					});
			return keys;
		}

		public boolean isWordStart(char c) 
		{
			return Character.isJavaIdentifierStart(c);
		}

		public boolean isWordPart(char c) 
		{
			if (c == '.') { return true; } 
			return Character.isJavaIdentifierPart(c); 
		}
	}
	
	static public class NumberDetector extends RMLBuiltinRelationsDetector implements IWordDetector
	{
        /**
         * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
         */
        public boolean isWordStart(char c)
        {
            return Character.isDigit(c) || c == '-' || c == '~' || c == '.'; 
        }

        /**
         * @see org.eclipse.jface.text.rules.IWordDetector#isWordPart(char)
         */
        public boolean isWordPart(char c) 
        {
            return Character.isDigit(c) || c == 'e'  || c == '.' || c == 'x';
        }
	}	
	
	static public class RMLAmpersantDetector extends NumberDetector implements IWordDetector
	{
        /**
         * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
         */
        public boolean isWordStart(char c) 
        {
            if (c == '&') { return true; }
            if (c == '[') { return true; }
            if (c == ']') { return true; }
            if (c == ':') { return true; }
            if (c == '=') { return true; }
            if (c == '*') { return true; }            
            if (c == '|') { return true; }             
            return false;
        }

        /**
         * @see org.eclipse.jface.text.rules.IWordDetector#isWordPart(char)
         */
        public boolean isWordPart(char c) 
        {
            if (c == ':') { return true; }        	
            if (c == '>') { return true; }        	            
            return false;
        }
	}	

	/**
	 * RML DATATYPE constructor detector
	 */	
	static public class RMLDataypeConstructorDetector extends RMLAmpersantDetector implements IWordDetector
	{
        /**
         * @see org.eclipse.jface.text.rules.IWordDetector#isWordStart(char)
         */
		public boolean isWordStart(char c) 
		{
			if (c == '_') { return false; }
			return Character.isJavaIdentifierStart(c) && Character.isUpperCase(c);
		}

		public boolean isWordPart(char c) 
		{
			if (c == '\'') { return true; }
			if (c == '_') { return true; }			
			if (Character.isDigit(c)) {return true; }
			return Character.isJavaIdentifierPart(c) && Character.isUpperCase(c);
		}
	}	
	
	/**
	 * RML identifier detector
	 */
	static private class RMLIdentifierDetector extends RMLDataypeConstructorDetector implements IWordDetector 
	{
		public RMLIdentifierDetector() 
		{
		}

		public boolean isWordStart(char c) 
		{
			if (c == '_') { return false; }
			return Character.isJavaIdentifierStart(c);
		}

		public boolean isWordPart(char c) 
		{
			if (c == '\'') { return true; }
			if (c == '.') { return true; }
			return Character.isJavaIdentifierPart(c);
		}
	}	
	
	public class WordRuleExtended extends WordRule implements IRule
	{
		
		public WordRuleExtended(IWordDetector detector)
		{
			super(detector);
		}
		
        public WordRuleExtended(IWordDetector detector, IToken defaultToken)
        {
        	super(detector, defaultToken);
        }
        
        public IToken evaluate(ICharacterScanner scanner)
        {
        	IToken token = super.evaluate(scanner);
        	int chr = scanner.read();
        	if (Character.isWhitespace(chr) || 
        	    (!Character.isLetterOrDigit(chr) && (char)chr != '.')) 
        	{
        		scanner.unread();
        		return token;
        	}
        	else 
        	{
        		scanner.unread();
        		return Token.UNDEFINED;
        	}
        }

	}	
	
	public RMLRuleScanner() 
	{
		super();
		setupRules();
	}

	private void setupRules()
	{
		IToken commentToken = 
			new Token(new TextAttribute(COMMENT_COLOR,null,SWT.NORMAL));
		IToken stringToken = 
			new Token(new TextAttribute(STRING_COLOR,null,SWT.NORMAL));
		IToken charToken = 
			new Token(new TextAttribute(CHAR_COLOR,null,SWT.NORMAL));
		IToken keywordToken = 
			new Token(new TextAttribute(KEYWORD_COLOR,null,SWT.BOLD));
		IToken defaultToken = 
			new Token(new TextAttribute(CODE_COLOR,null,SWT.NORMAL));
		IToken typeToken = 
			new Token(new TextAttribute(TYPE_COLOR,null,SWT.BOLD));
		IToken typeVarToken = 
			new Token(new TextAttribute(TYPEVAR_COLOR,null,SWT.BOLD));
		IToken numberToken = 
			new Token(new TextAttribute(NUMBER_COLOR,null,SWT.NORMAL));
		IToken relationToken = 
			new Token(new TextAttribute(BUILTIN_COLOR,null,SWT.BOLD));
		IToken errorToken = 
			new Token(new TextAttribute(ERROR_COLOR,null,SWT.BOLD));
		IToken dashesToken = 
			new Token(new TextAttribute(DASHES_COLOR,null,SWT.NORMAL));
		IToken ampersantToken = 
			new Token(new TextAttribute(AMPERSANT_COLOR,null,SWT.NORMAL));
		IToken datatypeConstructorToken = 
			new Token(new TextAttribute(DATATYPECONSTRUCTOR_COLOR,null,SWT.BOLD));
		
		setDefaultReturnToken(errorToken);
		
		List<IRule> rules = new ArrayList<IRule>();
		
		rules.add(new WhitespaceRule(new WhiteSpace()));
		
		WordRule wordRuleKeyword = new WordRule(new RMLKeywordDetector());
		
		for (int i=0; i< RMLKeywordDetector.keywords.length;i++) 
		{
			wordRuleKeyword.addWord(RMLKeywordDetector.keywords[i], keywordToken);
		}
		
		WordRule wordRuleType = new WordRule(new RMLTypeDetector());		
		for (int i=0; i< RMLTypeDetector.keywords.length;i++) 
		{
			wordRuleType.addWord(RMLTypeDetector.keywords[i], typeToken);
		}
		
		WordRule wordRuleBuiltinRelation = new WordRule(new RMLBuiltinRelationsDetector());		
		Vector<String> keys = RMLBuiltinRelationsDetector.getSortedKeywordsList();
		for (int i=0; i< keys.size(); i++) 
		{
			wordRuleBuiltinRelation.addWord(keys.elementAt(i), relationToken);
		}		
		
		
		WordRule wordRuleAmpersant = new WordRule(new RMLAmpersantDetector());		
		wordRuleAmpersant.addWord("&", ampersantToken);
		wordRuleAmpersant.addWord("[", ampersantToken);
		wordRuleAmpersant.addWord("]", ampersantToken);
		wordRuleAmpersant.addWord("=>", ampersantToken);	
		wordRuleAmpersant.addWord("=", ampersantToken);	
		wordRuleAmpersant.addWord("::", ampersantToken);
		wordRuleAmpersant.addWord(":", ampersantToken);		
		wordRuleAmpersant.addWord("*", ampersantToken);		
		wordRuleAmpersant.addWord("|", ampersantToken);
		
		WordRule wordRuleNumber = new WordRule(new NumberDetector(), numberToken);
		WordRule wordRuleTypeVar = new WordRule(new RMLTypeVariableDetector(), typeVarToken);		
		WordRule wordRuleDatatypeConstructor = 
			new WordRuleExtended(new RMLDataypeConstructorDetector(), datatypeConstructorToken);		
		WordRule wordRuleIdentifier = new WordRule(new RMLIdentifierDetector(), defaultToken);
		
		rules.add(wordRuleNumber);
		rules.add(wordRuleTypeVar);		
		rules.add(wordRuleBuiltinRelation);
		rules.add(wordRuleKeyword);
		rules.add(wordRuleType);
		rules.add(wordRuleAmpersant);
		rules.add(wordRuleDatatypeConstructor);
		rules.add(wordRuleIdentifier);
		rules.add(new SingleLineRule( "--", null, dashesToken));
		rules.add(new MultiLineRule( "\"", "\"", stringToken,'\\'));
		rules.add(new MultiLineRule( "#\"", "\"", charToken,'\\'));
		rules.add(new EndOfLineRule("//", commentToken,'\\'));
		rules.add(new MultiLineRule("/*", "*/", commentToken, (char) 0, true));
		rules.add(new MultiLineRule("(*", "*)", commentToken, (char) 0, true));
		
		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}
}
